<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 06/02/2011 *
 * ===================                          Last change: 06/02/2011 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : _filter.php                                      *
 * -------------------------------------------------------------------- *
 * Short description : Filters for ext-                                 *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Filter fuer ext-                                 *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Filter for adding login bonus to the user's account
function FILTER_ADD_LOGIN_BONUS ($filterData) {
	// Is the user data valid?
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called!');
	if (!isMember()) {
		// Do only run for logged in members
		reportBug(__FUNCTION__, __LINE__, 'Please only run this filter for logged in users.');
	} // END - if

	// Bonus is not given by default ;-)
	$bonus = FALSE;
	if ((isExtensionInstalledAndNewer('sql_patches', '0.2.8')) && (isBonusRallyeActive()) && (isIncludeBonusLoginEnabled())) {
		// Update last login if far enougth away
		// @TODO This query isn't right, it will only update if the user was for a longer time away!
		SQL_QUERY_ESC('UPDATE
	`{?_MYSQL_PREFIX?}_user_data`
SET
	`last_login`=UNIX_TIMESTAMP()
WHERE
	`userid`=%s AND
	 (UNIX_TIMESTAMP() - `last_login`) >= {?login_timeout?}
LIMIT 1',
			array(
				getMemberId()
			), __FUNCTION__, __LINE__
		);

		// Updated entry?
		$bonus = (!SQL_HASZEROAFFECTED());
	} // END - if

	if (($bonus === TRUE) && (getRequestElement('do') == 'bonus')) {
		// Output message with added points
		$GLOBALS['message'] .= '<div class="tiny">{--MEMBER_BONUS_LOGIN_BONUS_ADDED--}</div>';
	} else {
		// No login bonus added!
		$GLOBALS['message'] .= '<div class="notice">{--MEMBER_BONUS_LOGIN_BONUS_NOT_ADDED--}</div>';
	}

	// Return data
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Done!');
	return $filterData;
}

// Filter for generating admin mail links for bonus mails
function FILTER_GENERATE_BONUS_MAIL_LINKS ($filterData) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called!');

	// Is the type 'bid'?
	if ($filterData['type'] == 'bid') {
		// Load template
		$filterData['__output'] .= loadTemplate('admin_links_bonus_mail', TRUE, $filterData);
	} // END - if

	// Return data
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Done!');
	return $filterData;
}

// Filter for adding bonus columns with a plus (+) sign
function FILTER_ADD_BONUS_POINTS_USER_COLUMNS ($add = '') {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called!');

	// Skip out-dated extension, because this causes an error
	if (isExtensionInstalledAndOlder('bonus', '0.9.7')) {
		// Skip this silently
		return $add;
	} // END - if

	// Add more columns only when the corresponding configuration is enabled, too
	if (isIncludeBonusClickEnabled()) $add .= ' + `turbo_bonus`';
	if (isIncludeBonusLoginEnabled()) $add .= ' + `login_bonus`';
	if (isIncludeBonusOrderEnabled()) $add .= ' + `bonus_order`';
	if (isIncludeBonusRefEnabled()  ) $add .= ' + `bonus_ref`';
	if (isIncludeBonusStatsEnabled()) $add .= ' + `bonus_stats`';

	// Return $add
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Done!');
	return $add;
}

// Filter for adding SQL columns on user registration for bonus/"active" rallye notification
function FILTER_BONUS_USER_REGISTRATION_ADD_SQL_COLUMNS ($filterData) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called!');

	// Next: active rallye
	if (!isBonusNewMemberNotifyEnabled()) {
		addExtraRegistrationColumns(', `bonus_rallye_enable_notify`, `bonus_rallye_disable_notify`');
		$GLOBALS['register_sql_data']    .= ', UNIX_TIMESTAMP(), UNIX_TIMESTAMP()';
	} // END - if

	// Return filter data
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Done!');
	//* NOISY-DEBUG: */ print __FUNCTION__.':filterData=<pre>'.print_r($filterData,TRUE).'</pre>';
	return $filterData;
}

// Filter for sending out bonus rallye notifications
function FILTER_SEND_BONUS_NOTIFICATIONS ($filterData) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Called!');
	// Check for bonus rallye is active and send mails out
	if ((isBonusRallyeActive()) && (isBonusNewMemberNotifyEnabled())) {
		// Include file for sending out mails
		addIncludeToPool('notify', 'inc/mails/bonus_mails.php');
	} // END - if

	// Return filter data
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Done!');
	//* NOISY-DEBUG: */ print __FUNCTION__.':filterData=<pre>'.print_r($filterData,TRUE).'</pre>';
	return $filterData;
}

// [EOF]
?>
