<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 02/17/2005 *
 * ===================                          Last change: 02/17/2005 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : doubler_functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Functions for the guest's newsletter             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer den Newsletter an die Gaeste     *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Generates a HTML table based on given data
// @TODO Lame description
function generateDoublerTable ($userid = NULL, $done = 'N', $ref = 'N', $sort = 'ASC') {
	$add = ''; $DT_MODE = '0';
	if (isValidUserId($userid)) {
		// Load entries only from a single user
		$add = sprintf(" AND `userid`=%s", bigintval($userid));
		$mode = 'member'; $COLS = 4; $DT_MODE = 2;
		$message = '{--MEMBER_DOUBLER_NO_ENTRIES_FOUND--}';
	} else {
		// Guest mode!
		$mode = 'guest'; $COLS = 3; $DT_MODE = 3;
		$message = '{--GUEST_DOUBLER_NO_ENTRIES_FOUND--}';
	}

	if (($done == 'Y') && ($sort == 'ASC')) {
		// Already payed out points (latest payouts first)
		$limit = getConfig('doubler_display_old');
	} elseif ($sort == 'ASC') {
		// List entries which will receive their payout soon
		$limit = getConfig('doubler_display_pay');
	} elseif ($sort == 'DESC') {
		// Newest entries
		$limit = getConfig('doubler_display_new');
	}

	// List entries
	$result = SQL_QUERY_ESC("SELECT
	`userid`,
	`refid`,
	`points`,
	`timemark`
FROM
	`{?_MYSQL_PREFIX?}_doubler`
WHERE
	`completed`='%s' AND
	`is_ref`='%s'
	" . $add . "
ORDER BY
	`timemark` %s
LIMIT %s",
		array(
			$done,
			$ref,
			$sort,
			$limit
		), __FUNCTION__, __LINE__);

	if (!SQL_HASZERONUMS($result)) {
		// List entries
		$OUT = '';
		while ($content = SQL_FETCHARRAY($result)) {
			// Rewrite userid/refid only if admin is in
			// @TODO Can't this be moved into EL?
			if (isAdmin()) {
				// Set empty userid/refid
				$content['userid'] = '---';
				$content['refid']  = '---';

				// Set links to admin area
				if (isValidUserId($content['userid'])) $content['userid'] = generateUserProfileLink($content['userid']);
				if (isValidUserId($content['refid']))  $content['refid']  = generateUserProfileLink($content['refid']);
			} // END - if

			// Prepare data for the row template
			$content['timemark'] = generateDateTime($content['timemark'], $DT_MODE);

			// Load template and switch color
			$OUT .= loadTemplate($mode . '_doubler_list_rows', TRUE, $content);
		} // END - while

		// Free memory
		SQL_FREERESULT($result);
	} else {
		// List no entries
		$OUT = '<tr>
  <td colspan="' . $COLS . '" align="center" class="bottom">
    ' . displayMessage($message, TRUE) . '
  </td>
</tr>';
	}

	// Return template
	return loadTemplate($mode . '_doubler_list', TRUE, $OUT);
}

// Get total points left in doubler pot
function getDoublerTotalPointsLeft() {
	// Initialize variables
	$points = '0';

	if (getConfig('doubler_own') == 'Y') {
		// Take points from doubler's own account
		$points += getDoublerPoints() - getConfig('doubler_used');
	} // END - if

	if ((getConfig('doubler_jackpot') == 'Y') && (isExtensionActive('jackpot'))) {
		// Load jackpot
		$jackpot = getJackpotPoints();

		if (!empty($jackpot)) $points += $jackpot;
	} // END - if

	if (isValidUserId(getDoublerUserid())) {
		// Get user's points
		$user = getTotalPoints(getDoublerUserid());
		$points += $user;
	} // END - if

	// Return value
	return $points;
}

//-----------------------------------------------------------------------------
//                      Wrapper functions for ext-doubler
//-----------------------------------------------------------------------------

// "Getter" for doubler_userid
function getDoublerUserid () {
	// Is it cached?
	if (!isset($GLOBALS['doubler_userid'])) {
		// Get it
		$GLOBALS['doubler_userid'] = getConfig('doubler_userid');
	} // END - if

	// Return cache
	return $GLOBALS['doubler_userid'];
}

// "Getter" for doubler_timeout
function getDoublerTimeout () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('doubler_timeout');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for doubler_send_mode
function getDoublerSendMode () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('doubler_send_mode');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for doubler_ref
function getDoublerRef () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('doubler_ref');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for doubler_points
function getDoublerPoints () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('doubler_points');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for doubler_min
function getDoublerMin () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('doubler_min');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for doubler_max
function getDoublerMax () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('doubler_max');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for doubler_counter
function getDoublerCounter () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('doubler_counter');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for doubler_charge
function getDoublerCharge () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('doubler_charge');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for doubler_max_sent
function getDoublerMaxSent () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('doubler_max_sent');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// [EOF]
?>
