/**
 * JavaScript for common installer functions
 * --------------------------------------------------------------------
 * $Revision::                                                        $
 * $Date::                                                            $
 * $Tag:: 0.2.1-FINAL                                                 $
 * $Author::                                                          $
 * --------------------------------------------------------------------
 * Copyright (c) 2003 - 2009 by Roland Haeder
 * Copyright (c) 2009 - 2012 by Mailer Developer Team
 * For more information visit: http://mxchange.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */

// Installation steps array
var installationSteps = new Array();

// Failed step
var failedStep = '';
var counterSuccess = 0;

// Init all installation steps
installationSteps[0] = 'import_tables_sql';
installationSteps[1] = 'import_menu_sql';
installationSteps[2] = 'install_extensions';

// Always keep as last step
installationSteps[installationSteps.length] = 'write_local_config';

// Switches instaler by redirecting
function switchInstaller (installer) {
	// Switch installer
	document.location.href = 'install.php?installer=' + installer;
}

// User has clicked on 'finish'
function doFinishInstallation () {
	// First disable all buttons button
	resetFooterNavigation();

	// Display progress window
	displayProgressWindow('install', 'Init ...');

	// Start installation loop delayed
	window.setTimeout('doInstallationLoop()', 500);

	// Wait here
	window.setTimeout('doWait()', 500);
}

// Do wait
function doWait () {
	// Is still something to do?
	if ((failedStep == '') && (counterSuccess != installationSteps.length)) {
		// Wait one round more
		window.setTimeout('doWait()', 500);
	} else if ((failedStep == '') && (counterSuccess == installationSteps.length)) {
		// Close window
		closeProgressWindow('install', true, false);

		// Redirect to old 'finished' page
		document.location.href = 'admin.php';
	} else if (failedStep != '') {
		// Something happens that should not happen!
		displayErrorWindow('install', 'failedStep=' + failedStep);
	}
}

// Update progress bar
function updateProgressBar () {
	// Increment counter
	counterSuccess++;

	// Do only update <= 100% values
	if (counterSuccess <= installationSteps.length) {
		// Update progress bar
		$('#progressbar').progressbar({
			value: (counterSuccess / installationSteps.length * 100)
		});
	} // END - if
}

// Does the "installation loop"
function doInstallationLoop () {
	// For-loop for all installation steps
	for (var i = 0; i < installationSteps.length; i++) {
		// Output message
		outputInstallationStepMessage(installationSteps[i]);

		// Initialize next step
		if (!sendInstallationStepRequest(i) == true) {
			// Failed step, so remember it for later display
			failedStep = installationSteps[i];

			// Stop here
			break;
		} // END - if

		// Wait a little
		$('body').delay(500);

		// Update progress bar
		updateProgressBar();
	} // END - for

	// Is success counter same as array size
	if (counterSuccess != installationSteps.length) {
		// Display error message
		displayErrorWindow('install', getAjaxContent() + ':' + counterSuccess + '/' + installationSteps.length + ':' + failedStep);
	} // END - if
}

// Sends an "installation step" request out
function sendInstallationStepRequest (i) {
	// Is it set?
	if (installationSteps[i] == undefined) {
		// Not set installation step, so don't send it out
		throw new 'installationSteps[' + i + '] is not set.';
	} // END - if

	// Send out the request
	return sendAjaxRequest('install', 'do_step', '&step=' + installationSteps[i], true);
}

// Outputs a "step message"
function outputInstallationStepMessage (step) {
	// Set content
	// @TODO Progress bar is out-of-order: + '<div id="progressbar"></div>'
	setProgressContent('install', step);
}
